"use strict";
var async_1 = require('../serenity/recording/async');
var withArityOf = require('util-arity');
/**
 * Monkey-patches Cucumber.js Given/When/Then step generators to ensure that any step definition they create
 * is executed within WebDriver's ControlFlow and therefore synchronised with it
 *
 * @param cucumber
 * @param controlFlow
 */
function synchronise(cucumber, controlFlow) {
    [
        'Given',
        'When',
        'Then',
    ]
        .forEach(function (stepGenerator) { return cucumber[stepGenerator] = synchronising(cucumber[stepGenerator]); });
    // ---
    /**
     * Creates a synchronising StepGenerator, which looks like a regular StepGenerator but with this signifficant
     * difference, that any step function passed to it will be wrapped and executed in the context of WebDriver
     * Control Flow
     *
     * @param originalStepGenerator
     * @return {StepGenerator}
     */
    function synchronising(originalStepGenerator) {
        function synchronisingStepGenerator(pattern, options, code) {
            var originalStep = code || options, synchronised = mimic(originalStep, synchronisedStep(originalStep));
            var params = !!code
                ? [pattern, options, synchronised]
                : [pattern, synchronised];
            return originalStepGenerator.apply(cucumber, params);
        }
        return mimic(originalStepGenerator, synchronisingStepGenerator);
    }
    /**
     * Provides a synchronised wrapper around the user-defined step
     *
     * @param originalStep
     * @return {(args:...[any])=>Promise<void>}
     */
    function synchronisedStep(originalStep) {
        return function () {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i - 0] = arguments[_i];
            }
            var deferred = new async_1.Deferred(), context = this;
            controlFlow
                .execute(function () { return originalStep.apply(context, args); })
                .then(deferred.resolve, deferred.reject);
            return deferred.promise;
        };
    }
}
exports.synchronise = synchronise;
/**
 * Makes the pretender function of the same arity as the original one to deceive cucumber.
 *
 * @param original
 * @param pretender
 * @return {(args:...[any])=>any}
 */
function mimic(original, pretender) {
    return withArityOf(original.length, pretender);
}

//# sourceMappingURL=webdriver_synchroniser.js.map
